#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import os
import platform
from abc import abstractmethod

from injector import provider, Module, Injector

from ..base import ITXO


class Env(ITXO):
    @abstractmethod
    def username(self) -> str:
        raise NotImplementedError()


class NTEnv(Env):
    def username(self) -> str:
        return os.getenv('USERNAME')


class OSXEnv(Env):
    def username(self) -> str:
        return os.getenv('USER')


class EnvModule(Module):
    _platforms = {
        'Windows': NTEnv,
        'Darwin': OSXEnv,
    }

    @provider
    def provide(self, platform_name: str = platform.system()) -> Env:
        clz = EnvModule._platforms.get(platform_name, None)
        return clz() if clz is not None else None


if __name__ == '__main__':
    inj = Injector([EnvModule])
    e1 = inj.get(Env)
    e2 = inj.get(Env)
    print(e1.__hash__(), e2.__hash__())
